### gen_object function ###

gen_object <- function(nclu = 4, nvar = 4, ncat = c(2,4,3,2), dist = 0.3, discretize = "interval", mem = 1) {
  
  # parameters
  
  # nclu - the required number of clusters
  # nvar - the required number of variables
  # ncat - a vector expressing the numbers of categories in each of the variables
  # dist - the minimal between-cluster distance (-1,1)
  # discretize - a method of discretization ("interval" = equal interval width; "frequency" = equal frequencies")
  
  library(clusterGeneration)
  library(arules)
  
  gen <- genRandomClust(numClust=nclu, sepVal=dist, numNonNoisy=nvar,numNoisy=0,                 
                        numOutlier=0, numReplicate=1, fileName="GEN", clustszind=2,                
                        clustSizeEq=100, rangeN=c(100,200), clustSizes=NULL, covMethod=c("eigen"),
                        rangeVar=c(1, 10), lambdaLow=1, ratioLambda=10, rotateind=TRUE, 
                        iniProjDirMethod=c("SL"), projDirMethod=c("newton"), alpha=0.05, 
                        ITMAX=20, eps=1.0e-10, quiet=TRUE, outputDatFlag=TRUE, 
                        outputLogFlag=FALSE, outputEmpirical=TRUE, outputInfo=FALSE)
  
  data <- data.frame(gen$datList)
  nvar <- length(data[1,])
  Nobjects <- length(data[,2])
  dis_data <- data.frame(matrix(0,nrow=Nobjects))
  
  for(i in 1:nvar)
  {
    dis_data[[paste("var",i,sep="")]] <- discretize(data[,i], method=discretize, categories = ncat[i], labels=c(1:ncat[i]), 
                                                    ordered=FALSE, onlycuts=FALSE)    
  }
  
  dis_data <- dis_data[,-1]
  dis_data <- data.matrix(dis_data)
  dis_data <- data.frame(dis_data)
  
  if (mem == 1) {
    mem <- read.csv("GEN_1.mem", header = F)
    names(mem) <- "mem"
    dis_data <- cbind(dis_data,mem)
  }
  
  file.remove("GEN_1.mem", "GEN_1.dat", "GEN_1.noisy")
  
  return(dis_data)
}


### DATASETS GENERATION ###

#.......................................................0. load libraries and set wd
library(TTR)
library(cluster)

setwd() #set working directory

#.......................................................1. load datasets features
properties=read.csv("datasets_features.csv", sep=";")

#.......................................................2. generate data
for(i in 1:810){
  
  n.var=properties[i,]$Variables #number of variables
  n.var
  
  var.names=c("VAR_1","VAR_2","VAR_3","VAR_4","VAR_5","VAR_6","VAR_7","VAR_8","VAR_9","VAR_10")
  var.names=var.names[1:n.var] #variables names
  var.names
  
  cat=vector() #number of categories for each variable
  for(j in 1:n.var){
    if(properties$Categories[i]=="s"){cat[j]=sample(2:4,1)}
    if(properties$Categories[i]=="m"){cat[j]=sample(5:7,1)}
    if(properties$Categories[i]=="l"){cat[j]=sample(8:10,1)}
  }
  cat  
  
  clu=properties[i,]$Clusters #number of clusters
  clu 
  
  dist=properties[i,]$Distance #distance between clusters
  dist 
  
  data=gen_object(nclu = clu, nvar = n.var, ncat = cat, dist = dist, discretize = "interval", mem = 1)
  data
  
  print(i)
  
  if(i<10){write.csv(data,paste("data00",i,".csv", sep = ""),row.names = F) }
  if(i>9 & i<100){write.csv(data,paste("data0",i,".csv", sep = ""),row.names = F) }
  if(i>99){write.csv(data,paste("data",i,".csv", sep = ""),row.names = F) }
  
}

#.......................................................3. save properties
properties
write.csv(properties,"datasets_features+clusters.csv",row.names = F) 
