setwd("...")

### BK index ###

bk_index = function(data,tree,max.k = 50){
  
  k = 2:max.k
  EE = numeric(max.k)
  EE[1] = entropy_bk(data)
  II = numeric(max.k)
  II[max.k] = NA
  for(i in k){
    data$cl = tree[,i-1]
    EE[i] = expected_entropy(data)
    II[i-1] = EE[i-1]-EE[i]
  }
  BK = numeric(max.k)
  BK[1] = NA
  BK[max.k] = NA
  BK[2:(max.k-2)] = (II[1:(max.k-3)]-II[2:(max.k-2)])-(II[2:(max.k-2)]-II[3:(max.k-1)])
  return(BK)
}

entropy_bk = function(data,norm=FALSE){
  entrop = sum(na.omit(apply(data,2,function(x) -sum(table(x)/length(x)*log(table(x)/length(x))/log(length(unique(x)))))))
  return(entrop)
}

entropy2 = function(data,norm=FALSE){
  entrop = sum(na.omit(apply(data,2,function(x) length(x)*sum(table(x)/length(x)*log(table(x)/length(x))))))
  return(entrop)
}

expected_entropy = function(data){
  cl.ind <- which(colnames(data)=="cl")
  n_j = table(data[,cl.ind])
  h_c <- numeric(length(n_j))
  for(i in 1:length(n_j)){
    h_c[i] <- entropy_bk(data[data[,cl.ind]==i,-cl.ind])
  }
  EE = sum(n_j*h_c)/sum(n_j)
  return(EE)
}

### AIC and BIC criteria ###

entropy = function(data,norm=FALSE){
  entrop = sum(na.omit(apply(data,2,function(x) length(x)*sum(table(x)/length(x)*log(table(x)/length(x))))))
  return(entrop)
}

mutability = function(data,norm=FALSE){
  mutab = (sum(na.omit(apply(data,2,function(x) length(x)*(1 -  sum(table(x)/length(x)*(table(x)/length(x))))))))
  return(mutab)
}

aic <- function(d,a){
  output <- data.frame("AIC1"=0,"AIC1_cl"=0,
                       "AIC2"=0,"AIC2_cl"=0,
                       "BIC1"=0,"BIC1_cl"=0,
                       "BIC2"=0,"BIC2_cl"=0)
  
  mem <- a$mem
  entr<- numeric(ncol(mem))
  mutab <- numeric(ncol(mem))
  for(f in 1:ncol(mem)){
    entr[f] <- 0
    mutab[f] <- 0
    for(w in 1:max(mem[,f])){
      entr[f] <- entr[f] + entropy(d[mem[,f]==w,])
      mutab[f] <- mutab[f] + mutability(d[mem[,f]==w,])
    }
  }
  
  kc <- sum(apply(d,2,function(x) length(unique(x))-1))
  
  aic1 <- -2*entr+2*(2:(ncol(mem)+1))*(kc)
  aic2 <- -2*mutab+2*(2:(ncol(mem)+1))*(kc)
  
  bic1 <- -2*entr+(2:(ncol(mem)+1))*(kc)*log(nrow(d))
  bic2 <- -2*mutab+(2:(ncol(mem)+1))*(kc)*log(nrow(d))
  
  lk <- c(entropy(d),entr)
  dk <- lk[-length(lk)]-lk[-1]
  dk[dk==0] <- 0.0001
  R <- dk[-length(dk)]/dk[-1] 
  R_s <- sort(R,decreasing = T)
  rat_entr <- R_s[1]/R_s[2]
  threshold <- 1.15
  
  if(rat_entr>threshold){
    output$AIC1_cl[1] <- which(R==R_s[1])+1
    output$AIC1[1] <- aic1[which(R==R_s[1])]
    output$BIC1[1] <- which(R==R_s[1])+1
    output$BIC1_cl[1] <- bic1[which(R==R_s[1])]
  } else{
    output$AIC1_cl[1] <- max(which(R==R_s[1]),which(R==R_s[1]))+1
    output$AIC1[1] <- aic1[which(R==R_s[1])]
    output$BIC1[1] <- max(which(R==R_s[1]),which(R==R_s[1]))+1
    output$BIC1_cl[1] <- bic1[which(R==R_s[1])]
  }
  lk <- mutab
  dk <- lk[-length(lk)]-lk[-1]
  dk[dk==0] <- 0.0001
  R <- dk[-length(dk)]/dk[-1] 
  R_s <- sort(R,decreasing = T)
  rat_mutab <- R_s[1]/R_s[2]
  threshold <- 1.15
  
  if(rat_entr>threshold){
    output$AIC2_cl[1] <- which(R==R_s[1])+2
    output$AIC2[1] <- aic2[which(R==R_s[1])+1]
    output$BIC2[1] <- which(R==R_s[1])+2
    output$BIC2_cl[1] <- bic2[which(R==R_s[1])+1]
    
  } else{
    output$AIC2_cl[1] <- which.min(aic2)+1
    output$AIC2[1] <- aic2[which.min(aic2)]
    output$BIC2[1] <- which.min(bic2)+1
    output$BIC2_cl[1] <- bic2[which.min(bic2)]
    
  }
  return(output)
}

### EVALUATION ###


library(nomclust)
library(NbClust)


measures <- c("sm", "eskin","iof", "lin", "ve")

linkage <- "single"    # other options are "complete" and "average"

options(warn=-1,message=-1)
files <- dir(pattern = "*.csv")
features <- files[grep("datasets_features",files)]


files <- files[grep("data",files)]
files <- files[-length(files)]
d <- read.csv(files[1])
feat <- read.csv(features)

output <- data.frame("file name"=rep(0,length(files)),"AIC1"=rep(0,length(files)),"AIC1_cl"=rep(0,length(files)),"AIC2"=rep(0,length(files)),"AIC2_cl"=rep(0,length(files)),
                     "BIC1"=rep(0,length(files)),"BIC1_cl"=rep(0,length(files)),"BIC2"=rep(0,length(files)),"BIC2_cl"=rep(0,length(files)),
                     "PSTau_max"=rep(0,length(files)),"PSTau_max_cl"=rep(0,length(files)),"PSTau_min_"=rep(0,length(files)),"PSTau_min_cl"=rep(0,length(files)),
                     "PSU_max"=rep(0,length(files)),"PSU_max_cl"=rep(0,length(files)),"PSU_min_"=rep(0,length(files)),"PSU_min_cl"=rep(0,length(files)),
                     "PSFM_max"=rep(0,length(files)),"PSFM_max_cl"=rep(0,length(files)),"PSFM_min_"=rep(0,length(files)),"PSFM_min_cl"=rep(0,length(files)),
                     "PSFE_max"=rep(0,length(files)),"PSFE_max_cl"=rep(0,length(files)),"PSFE_min_"=rep(0,length(files)),"PSFE_min_cl"=rep(0,length(files)),
                     "BK_max"=rep(0,length(files)),"BK_max_cl"=rep(0,length(files)),
                     "sil_max"=rep(0,length(files)),"sil_ax_cl"=rep(0,length(files)),
                     "dunn_max"=rep(0,length(files)),"dunn_ax_cl"=rep(0,length(files)),
                     "cind_max"=rep(0,length(files)),"cind_ax_cl"=rep(0,length(files)),
                     "McC_max"=rep(0,length(files)),"McC_ax_cl"=rep(0,length(files)),
                     "frey_max"=rep(0,length(files)),"frey_ax_cl"=rep(0,length(files)),
                     "variables"=rep(0,length(files)),"kategorie"=rep(0,length(files)),"clusters"=rep(0,length(files)),"size"=rep(0,length(files)))
output$file.name <- files
output$variables <- feat$Variables
output$kategorie <- feat$Categories
output$clusters <- feat$Clusters
output$size <- feat$Size

for(k in 1:length(measures)){
j=1
for(i in 1:length(files)){
  d <- read.csv(files[j])
  d <- d[,-ncol(d)]
  diss_matice <- get(measures[k])(d)

  write.csv2(diss_matice,paste(measures[k],"_",j,"_dist_mat.csv",sep = ""),row.names = F)
  
  diss_matice <- read.csv2(paste(measures[k],"_",j,"_dist_mat.csv",sep = ""))
  diss_matice <- as.dist(diss_matice)
  a <- nomprox(data = d, prox_matrix = diss_matice,clu_high = 10, method=linkage)
  
  
  output[j,10] <- max(a$eval[,4],na.rm = T)
  output[j,11] <- which.max(a$eval[,4])
  output[j,12] <- min(a$eval[,4],na.rm = T)
  output[j,13] <- which.min(a$eval[,4])
  
  output[j,14] <- max(a$eval[,5],na.rm = T)
  output[j,15] <- which.max(a$eval[,5])
  output[j,16] <- min(a$eval[,5],na.rm = T)
  output[j,17] <- which.min(a$eval[,5])  
  
  output[j,18] <- max(a$eval[,6],na.rm = T)
  output[j,19] <- which.max(a$eval[,6])
  output[j,20] <- min(a$eval[,6],na.rm = T)
  output[j,21] <- which.min(a$eval[,6])  
  
  output[j,22] <- max(a$eval[,7],na.rm = T)
  output[j,23] <- which.max(a$eval[,7])
  output[j,24] <- min(a$eval[,7],na.rm = T)
  output[j,25] <- which.min(a$eval[,7])
  
  tree <- a
  tree <- tree$mem
  t <- bk_index(d,tree = tree,max.k=8)
  output[j,26] <- max(t,na.rm = T)
  output[j,27] <- which.max(t) 
  
  ic <- aic(d,a)
  
  output$AIC1[j] <- ic$AIC1
  output$AIC1_cl[j]  <- ic$AIC1_cl
  output$AIC2[j]  <- ic$AIC2
  output$AIC2_cl[j]  <- ic$AIC2_cl
  output$BIC1[j]  <- ic$BIC1
  output$BIC1_cl[j]  <- ic$BIC1_cl
  output$BIC2[j]  <- ic$BIC2
  output$BIC2_cl[j]  <- ic$BIC2_cl
  

  vystup <- try(NbClust(data = NULL, diss = diss_matice, distance = NULL,
                        min.nc = 2, max.nc = 8, method = linkage, index = "silhouette")$Best.nc,silent=T)
  if(inherits(x = vystup,what = "try-error")) vystup = c(NA,NA)


  output[j,28] <- vystup[1]
  output[j,29] <- vystup[2]
  
  vystup <- try(NbClust(data = NULL, diss = diss_matice, distance = NULL,
                    min.nc = 2, max.nc = 8, method = linkage, index = "dunn")$Best.nc,silent=T)
  if(inherits(x = vystup,what = "try-error")) vystup = c(NA,NA)
  
  output[j,30] <- vystup[1]
  output[j,31] <- vystup[2]
  
  vystup <- try(NbClust(data = NULL, diss = diss_matice, distance = NULL,
                        min.nc = 2, max.nc = 8, method = linkage, index = "cindex")$Best.nc,silent=T)
  if(inherits(x = vystup,what = "try-error")) vystup = c(NA,NA)

  output[j,32] <- vystup[1]
  output[j,33] <- vystup[2]
  
  vystup <- try(NbClust(data = NULL, diss = diss_matice, distance = NULL,
                        min.nc = 2, max.nc = 8, method = linkage, index = "mcclain")$Best.nc,silent=T)
  if(inherits(x = vystup,what = "try-error")) vystup = c(NA,NA)

  
  output[j,34] <- vystup[1] 
  output[j,35] <- vystup[2]
  
  vystup <- try(NbClust(data = NULL, diss = diss_matice, distance = NULL,
                        min.nc = 2, max.nc = 8, method = linkage, index = "frey")$Best.nc,silent=T)
  if(inherits(x = vystup,what = "try-error")) vystup = c(NA,NA)

  
  output[j,36] <- vystup[1]
  output[j,37] <- vystup[2]
  
  j=1+j
  if(j%%30==0|j<3) print(j) 
}

write.csv2(output,paste(measures[k],"_output.csv",sep = ""))}
